
#include <nokix/all.h>

/*
** chooser.c
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "global.h"

extern rexx_uint confid_main;
extern rexx_uint confid_negative;
extern rexx_uint confid_skin;
extern rexx_uint num_of_skins;

/**************************************************************************************/

void ss_draw(void);

static void next_frame(void) /* anim timer handler */
{
 /* this little function is called when anim frame timer reaches 0
    we send here a message to tell our chooser dispatcher to draw next
	animation frame. we send current skin index as an argument, thanks
	to this chooser will be able to detect a situation when message
	was sent and didn't arrived before skin change, such a "old"
	messages are ignored then */
 send_ui_message(OMSG_NEXT_FRAME|MSG_1P,data->skin);
}

/* UI messages dispatcher */
extern rexx_uint chooser_id; /* dispatcher's call id */
rexx("chooser_id=add_5e0_id(chooser)"); /* auto creation of the id */

ULONG chooser(ULONG msg)
{
 switch(msg)
 {
  case MSG_D_INIT:
  {
   data=malloc(sizeof(struct data));
   data->skin=own_get_config(confid_skin);
   if(data->skin>=num_of_skins) data->skin=0;
   data->negative=own_get_config(confid_negative);

   own_screen_set(data->screen,0);
   freegfx_open();

   if(skins[data->skin]->flags&CIF_BACKGROUND_ANIM)
   {
    own_timer_start(next_frame,TT_FUNCTION,(((struct own_anim_frame *)(skins[data->skin]->background))[0]).time);
    data->frame=0;
   }

   ss_draw();

   msg=MSG_R_STAY;
   break;
  }
  case MSG_D_QUIT:
  case MSG_D_QUIT_NOW:
  {
   freegfx_close();
   own_timer_abort(next_frame);
   free(data); data=NULL;
   
   if(msg==MSG_D_QUIT) msg=MSG_R_QUIT;
   break;
  }
  case MSG_TIMESTAMP: /* comes every minute */
  {
   LONG t;

   if(skins[data->skin]->flags&CIF_BACKGROUND_ANIM)
   {
    t=own_timer_query(next_frame);
    if((t>=0)&&(t<=75)) break; /* less than half second to next frame -> don't bother */
   }
   ss_draw();
   break;
  }
  case OMSG_NEXT_FRAME:
  {
   if(msg_argv[0]==data->skin) /* check if this is for current skin, igonore if for old one */
   {
    if(!(((struct own_anim_frame *)(skins[data->skin]->background))[++data->frame]).bitmap) data->frame=0;

    own_timer_start(next_frame,TT_FUNCTION,(((struct own_anim_frame *)(skins[data->skin]->background))[data->frame]).time);

	ss_draw(); /* draw next frame */
   }
   msg=MSG_NOP;
   break;
  }
  case MSG_KEY_UP:
  case MSG_KEY_DOWN:
  {
   if(msg==MSG_KEY_UP) /* prevoius skin */
   {
    if(data->skin==0) data->skin=num_of_skins-1;
    else data->skin--;
   }
   else /* next skin */
   {
    if(data->skin==num_of_skins-1) data->skin=0;
    else data->skin++;
   }

   own_screen_set(data->screen,0);

   own_timer_abort(next_frame);
   if(skins[data->skin]->flags&CIF_BACKGROUND_ANIM)
   {
    own_timer_start(next_frame,TT_FUNCTION,(((struct own_anim_frame *)(skins[data->skin]->background))[0]).time);
    data->frame=0;
   }

   ss_draw();

   msg=MSG_NOP;
   break;
  }
  case MSG_KEY_ACTION1:
  {
   own_set_config(confid_main,1);
   own_set_config(confid_skin,data->skin);
   send_message(MSG_REQUEST_QUIT);
   send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_DONE,str_done);
   
   msg=MSG_NOP;
   //break;
  }
 }
 return msg;
}

