
#include <nokix/all.h>

/*
** ss_draw.c
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "global.h"

extern const UBYTE clock_type; /* 12/24 hour clock setting */
extern const UBYTE minutes_tab[][2];
extern const UBYTE hours_tab[][2];

extern LONG calc_battery_temp (void);

static const struct datetime dt_example=
{
 2005,1,12,
 22,10,0
};

/**************************************************************************************/

void ss_draw(void)
{
 struct datetime dt;
 LONG stamp=get_timestamp();
 void *font;
 char buf[40];
 UBYTE ampm;
 wchar_t *date;
 const nokstr_t *str;
 LONG x;
 struct line line;
 struct skin *skin=skins[data->skin];

 /* shouldn't happen :)) */
 if(!data) return;

 /* skin pointer */
 skin=skins[data->skin];
 
 /* start freegfx refreshing */
 freegfx_begin(FALSE);

 if((skin->flags&CIF_BACKGROUND_ANIM)&&(data->frame==0)&&(data->power_save)&&(skin->flags&CIF_POWER_SAVE_CLEAR))
 {
  /* we don't draw the background (but we clear the screen) when we enter power save mode and 
  CIF_POWER_SAVE_CLEAR skin flag is set */
  own_screen_set(freegfx_buffer(),0);
 }
 else
 {
  /* BACKGROUND */
  /* ---------- */

  /* we are copying our screen buffer back into freegfx screen buffer. our buffer is cleared for static backgrounds
  and for first draw of first frame of animated background */
  own_screen_copy(freegfx_buffer(),data->screen);
  if(skin->background)
  {
   if(skin->flags&CIF_BACKGROUND_ANIM) /* animation */
   {
    struct own_anim_frame *f=&(((struct own_anim_frame *)(skin->background))[data->frame]);

    own_draw_bitmap(f->bitmap,skin->background_x+f->left,skin->background_y+f->top,DM_SET);
    /* here we are copying this frame into our screen buffer so we can draw it back on next frame; this is usefull
    when we have "changes only" animation; we could use freegfx buffering but here we have it without clock yet
    so there's no "how to clear clock" problem */
    own_screen_copy(data->screen,freegfx_buffer());
   }
   else /* static image */
   {
    own_draw_bitmap((struct bitmap *)skin->background,skin->background_x,skin->background_y,DM_SET);
   }
  }
 }

 if(stamp==TIMESTAMP_UNSET) memcpy(&dt,&dt_example,sizeof(struct datetime));
 else own_extract_datetime(&dt,stamp);

 if(skin->flags&CIF_DIGITAL)
 {
  /* DIGITAL CLOCK */
  /* ------------- */

  /* 24/12 clock type play */
  x=dt.hours;
  if(clock_type_12h) /* if in 12-hours mode */
  {
   if(x>=12) { ampm='.'; x-=12; }
   else ampm=0x27; // apostrof
   if(x==0) x=12;
  }
  else ampm=0;

  /* long to ascii conversion */
  ntoa2(x,buf);
  buf[2]=':';
  ntoa2(dt.mins,buf+3);
  buf[5]=ampm;
  buf[6]=0;

  font=own_get_font(skin->digital_font);
  x=own_get_string_width((const nokstr_t *)buf,font);
  if(skin->flags&CIF_DIGITAL_RIGHT) x=skin->digital_x-x;
  else if(skin->flags&CIF_DIGITAL_CENTER) x=skin->digital_x-(x/2);
  else x=skin->digital_x;
  own_print_string((const nokstr_t *)buf,x,skin->digital_y,font,(skin->flags&CIF_DIGITAL_XOR)?DM_XOR:DM_OR);
 }

 if(skin->flags&CIF_WEEKDAY)
 {
  /* WEEKDAY NAME */
  /* ------------ */

  x=own_extract_weekday(stamp);
  if(skin->flags&CIF_WEEKDAY_SHORT) str=&str_short_weekdays[x];
  else str=&str_weekdays[x];
  font=own_get_font(skin->weekday_font);
  x=own_get_string_width(str,font);
  if(skin->flags&CIF_WEEKDAY_RIGHT) x=skin->weekday_x-x;
  else if(skin->flags&CIF_WEEKDAY_CENTER) x=skin->weekday_x-(x/2);
  else x=skin->weekday_x;
  own_print_string(str,x,skin->weekday_y,font,(skin->flags&CIF_WEEKDAY_XOR)?DM_XOR:DM_OR);
 }

 if(skin->flags&CIF_DATE)
 {
  /* DATE */
  /* ---- */

  if((skin->flags&CIF_DATE_DDMMYYYY)||(skin->flags&CIF_DATE_DDMM))
  {
   ntoa2(dt.day,buf);
   buf[2]='.';
   ntoa2(dt.month,buf+3);
   if(skin->flags&CIF_DATE_DDMMYYYY) /* if not VERY_SHORT then add year */
   {
    buf[5]='.';
    ntoa(dt.year,buf+6);
   }
   date=(WCSPTR)buf; /* buf isn't wcs but string functions will detect it */
  }
  else
  {
   ntoa(dt.year,buf);
   date=(wchar_t *)w_sprintf(str_datetime_formats[dt.month-1],dt.day,buf,0,0,0);
   for(x=0;date[x];x++) if(date[x]==10) break;
   date[x]=0;
  }

  font=own_get_font(skin->date_font);
  x=own_get_string_width((const nokstr_t *)date,font);
  if(skin->flags&CIF_DATE_RIGHT) x=skin->date_x-x;
  else if(skin->flags&CIF_DATE_CENTER) x=skin->date_x-(x/2);
  else x=skin->date_x;
  own_print_string((const nokstr_t *)date,x,skin->date_y,font,(skin->flags&CIF_DATE_XOR)?DM_XOR:DM_OR);
 }

 if(skin->flags&CIF_ANALOG)
 {
  /* ANALOG CLOCK */
  /* ------------ */
 
  /* line drawing struct initialization */
  line.pattern=NULL;
  line.x1=skin->analog_x;
  line.y1=skin->analog_y;
  line.dmode=(skin->flags&CIF_ANALOG_XOR)?DM_XOR:DM_OR;

  /* minutes hand */
  line.x2=line.x1+minutes_tab[dt.mins][0]-18;
  line.y2=line.y1+minutes_tab[dt.mins][1]-16;
  line.thickness=1;
  draw_line(&line);

  /* hours hand */
  x=dt.hours;
  if(x>=12) x-=12;
  x=5*x+dt.mins/12;
  line.x2=line.x1+hours_tab[x][0]-18;
  line.y2=line.y1+hours_tab[x][1]-16;
  line.thickness=2;
  draw_line(&line);
 }

 if(skin->flags&CIF_TEMPERATURE)
 {
  /* TEMPERATURE */
  /* ----------- */

  x = calc_battery_temp ();

  strcpy (buf, " xx'C");

  // convert to Fahrenheit
  if (skin->flags&CIF_TEMPERATURE_UNIT) {
  	x = (x * 18) / 5 + 64;		// Convert
  	x = (x >> 1) + (x & 1);		// Round
  	buf[4] = 'F';
  }
  // set the sign
  if (x > 0) {
	buf[0] = '+';
  } else if (x < 0) {
	x = -x;
  	buf[0] = '-';
  }

  // long to ascii conversion
  ntoa2 (x, &buf[1]);

  // move sign if one-digit value
  if (buf[1] == '0') {
	buf[1] = buf[0];
	buf[0] = ' ';
  }

  if (!(skin->flags&CIF_TEMPERATURE_SHORT)) buf[3] = '\'';  //'
  
  font=own_get_font(skin->temp_font);
  x=own_get_string_width((const nokstr_t *)buf,font);
  if(skin->flags&CIF_TEMPERATURE_RIGHT) x=skin->temp_x-x;
  else if(skin->flags&CIF_TEMPERATURE_CENTER) x=skin->temp_x-(x/2);
  else x=skin->temp_x;
  own_print_string((const nokstr_t *)buf,x,skin->temp_y,font,(skin->flags&CIF_TEMPERATURE_XOR)?DM_XOR:DM_OR);
 }

 /* invert screen contents if requiered */
 if(data->negative) own_draw_box(0,0,84,48,DM_XOR);

 /* end of refreshing
    -----------------
	we update LCD contents immediately (TRUE) beacause on 5110 UI task sometimes
	have no time to do this on fast animations */
 freegfx_end(TRUE);
}
